﻿package supermotion3d
{
	import flash.display.Sprite;
	import flash.display.BitmapData;
	import flash.display.TriangleCulling;
	import flash.geom.Matrix3D;
	import flash.geom.Vector3D;
	import flash.geom.Utils3D;

	public class BoneModel3D extends Model3D
	{
		public function BoneModel3D(sprite:Sprite = null):void
		{
			super(sprite);
		}

		override protected function init(texture:BitmapData):void
		{
			var b:int = 0;
			var i:int = 0;
			var j:int = 0;
			var index:int = 0;
			var indices:Vector.<int> = new Vector.<int>();

			//テクスチャ画像
			_image = texture;

			//ボーン数
			for ( i = 0; i < _boneNum; i++ )
			{
				_vertices3D.push(new Vector.<Number>());
				_uvtData.push(new Vector.<Number>());
				indices.push(0);
			}
			//ボーンごとに頂点とUV座標
			for ( i = 0; i < _indices.length; i++ )
			{
				b = _boneIndices[_indices[i]];
				_vertices3D[b].push(
					_vertices3D2[_indices[i]*3  ],
					_vertices3D2[_indices[i]*3+1],
					_vertices3D2[_indices[i]*3+2]);
				_uvtData[b].push(
					_uvtData2[i*3  ],
					_uvtData2[i*3+1],
					_uvtData2[i*3+2]);
			}
			//インデックス
			for ( i = 0; i < _indices.length; i++ )
			{
				var length:int = 0;
				b = _boneIndices[_indices[i]];
				for ( j = 0; j < b; j++ )
				{
					length += _vertices3D[j].length/3;
				}
				_indices2.push(indices[b]+length);
				indices[b] += 1;
			}
			_currentAnimation = 0;
		}

		protected function addVertexBone(x:Number,y:Number,z:Number,i:int):void
		{
			_vertices3D2.push(x,y,z);
			_boneIndices.push(i);
			if ( _boneNum <= i )
			{
				_boneNum = i + 1;
			}
		}

		override protected function addUV(u:Number,v:Number):void
		{
			_uvtData2.push(u,v,0);
		}

		protected function addIndex(i0:int,i1:int,i2:int):void
		{
			_indices.push(i0,i1,i2);
		}

		override public function draw(camera:Matrix3D,projection:Matrix3D):void
		{
			var i:int;
			var indices:Vector.<int> = new Vector.<int>();
			var vertices3D:Vector.<Number> = new Vector.<Number>();
			//2D変換後の頂点
			var vertices2D:Vector.<Number> = new Vector.<Number>();
			var uv:Vector.<Number> = new Vector.<Number>();
			var faces:Array = new Array();
			var faces2:Array = new Array();

			//画像消去
			this.graphics.clear();

			for ( var idx:int = 0; idx < _boneNum; idx++ )
			{
				if ( _vertices3D[idx].length > 0 )
				{
					//内部で使用する頂点
					var vert3D:Vector.<Number> = new Vector.<Number>();
					//2D変換後の頂点
					var vert2D:Vector.<Number> = new Vector.<Number>();
					//カメラ行列
					var m:Matrix3D = camera.clone();

					//変形行列
					m.prepend(_matrix);
					//アニメーション行列
					m.prepend(_animation[_currentAnimation][idx][_time]);
					//座標変換
					m.transformVectors(_vertices3D[idx], vert3D);
					Utils3D.projectVectors(projection, vert3D, vert2D, _uvtData[idx]);
					vertices2D = vertices2D.concat(vert2D);
					uv = uv.concat(_uvtData[idx]);
					vertices3D = vertices3D.concat(vert3D);
				}
			}
			//Z値を追加
			for ( i = 0; i < _indices2.length; i += 3 )
			{
				faces.push(
					(vertices3D[_indices2[i  ]*3+2]+
					 vertices3D[_indices2[i+1]*3+2]+
					 vertices3D[_indices2[i+2]*3+2])/3);
			}
			// Zソート (奥のものから順番に並べる)
			faces2 = faces.sort(Array.DESCENDING | Array.NUMERIC | Array.RETURNINDEXEDARRAY);
			for each( i in faces2 )
			{
				if ( faces[i] > 200 )
				{
					indices.push(_indices2[i*3],_indices2[i*3+1],_indices2[i*3+2]);
				}
			}

			if (indices.length > 0)
			{
				//マテリアル
				this.graphics.beginBitmapFill(_image);
				//3D描画
				this.graphics.drawTriangles(vertices2D, indices, uv, TriangleCulling.POSITIVE);
				this.graphics.endFill();
			}
		}
	}
}
